package com.gic.store.service.outer.impl;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.constant.SignConstants;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.error.ErrorCode;
import com.gic.store.constant.Constants;
import com.gic.store.constant.StoreOwnTypeEnum;
import com.gic.store.dto.StoreFieldDTO;
import com.gic.store.dto.StoreFieldRegionMapDTO;
import com.gic.store.dto.StoreFieldRegionRelDTO;
import com.gic.store.dto.StoreFieldRelDTO;
import com.gic.store.entity.TabStoreField;
import com.gic.store.entity.TabStoreFieldRel;
import com.gic.store.service.*;
import com.gic.store.utils.StoreFieldValidParamUtil;
import com.gic.store.utils.valid.ValidUtil;
import org.apache.commons.lang.math.NumberUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import java.util.*;

/**
 * @author guojx
 * @date 2019/6/28 1:40 PM
 */
@Service("storeFieldApiService")
public class StoreFieldApiServiceImpl implements StoreFieldApiService {
    private static final Logger LOGGER = LogManager.getLogger(StoreFieldApiServiceImpl.class);
    @Autowired
    private StoreFieldService storeFieldService;
    @Autowired
    private StoreExtendService storeExtendService;
    @Autowired
    private StoreRegionService storeRegionService;
    @Autowired
    private StoreFieldRelApiService storeFieldRelApiService;
    @Autowired
    private StoreFieldRelService storeFieldRelService;
    @Autowired
    private StoreFieldRegionRelService storeFieldRegionRelService;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public ServiceResponse<Integer> save(StoreFieldDTO storeFieldDTO) {
        ServiceResponse validParam = ValidUtil.allCheckValidate(storeFieldDTO, StoreFieldDTO.SaveFieldValidView.class);
        if (!validParam.isSuccess()) {
            return validParam;
        }

        if (StringUtils.isNotBlank(storeFieldDTO.getStoreFieldCode())) {
            if (storeFieldService.isRepeatCode(storeFieldDTO.getEnterpriseId(), null,
                    storeFieldDTO.getStoreFieldCode())) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "Code不能重复");
            }
        }
        if (storeFieldService.isRepeatName(storeFieldDTO.getEnterpriseId(), null, storeFieldDTO.getStoreFieldName())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "属性名称不能重复");
        }

        if (storeFieldService.countStoreField(storeFieldDTO.getEnterpriseId(), null) >= Constants.STORE_EXPORT_MAX_SIZE) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "自定义属性不能超过20个");
        }

        String[] regionArr = storeFieldDTO.getStoreRegionId().split("_");
        for (String region : regionArr) {
            if (StringUtils.isBlank(region)) {
                continue;
            }
            if (!StringUtils.isNumeric(region)) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "门店域有误");
            }
            if (storeRegionService.getById(storeFieldDTO.getEnterpriseId(), Integer.parseInt(region)) == null) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "门店域有误");
            }
        }
        ServiceResponse validParamResult = StoreFieldValidParamUtil
                .validStoreFieldDetail(storeFieldDTO.getStoreFieldType(), storeFieldDTO.getStoreFieldDetail());
        if (!validParamResult.isSuccess()) {
            return validParamResult;
        }
        storeFieldDTO.setCreateTime(new Date());
        storeFieldDTO.setUpdateTime(new Date());
        storeFieldDTO.setDeleteFlag(0);
        storeFieldDTO.setSort(storeFieldService.getMinSort(storeFieldDTO.getEnterpriseId()) - 1);
        int storeFieldId = storeFieldService.save(storeFieldDTO);

        List<StoreFieldRegionRelDTO> regionRelDTOList = new ArrayList<>();
        Date now = new Date();
        for (String regionId : regionArr) {
            if (StringUtils.isBlank(regionId)) {
                continue;
            }
            StoreFieldRegionRelDTO fieldRegionRelDTO = new StoreFieldRegionRelDTO();
            fieldRegionRelDTO.setCreateTime(now);
            fieldRegionRelDTO.setUpdateTime(now);
            fieldRegionRelDTO.setDeleteFlag(0);
            fieldRegionRelDTO.setRegionId(Integer.parseInt(regionId));
            fieldRegionRelDTO.setStoreFieldId(storeFieldId);
            regionRelDTOList.add(fieldRegionRelDTO);
        }
        //新增门店域门店属性光关联
        storeFieldRegionRelService.deleteByStoreFieldId(storeFieldId);
        storeFieldRegionRelService.saveFetch(regionRelDTOList);

        //新增企业关联表数据
        StoreFieldRelDTO storeFieldRelDTO = new StoreFieldRelDTO();
        storeFieldRelDTO.setEnterpriseId(storeFieldDTO.getEnterpriseId());
        storeFieldRelDTO.setStoreFieldId(storeFieldId);
        ServiceResponse storeFieldRelResult = storeFieldRelApiService.saveStoreFieldRel(storeFieldRelDTO);
        if (!storeFieldRelResult.isSuccess()) {
            //回滚
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return ServiceResponse.failure(storeFieldRelResult.getCode(), storeFieldRelResult.getMessage());
        }

        return ServiceResponse.success(storeFieldId);
    }

    @Override
    public ServiceResponse<Integer> upOrDown(int storeFieldId, boolean isUp, Integer enterpriseId) {
        TabStoreField storeField = storeFieldService.getStoreFieldById(storeFieldId);
        if (storeField == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "主键错误，查询不到数据");
        }
        StoreFieldDTO needUpdateSort;
        if (isUp) {
            needUpdateSort = storeFieldService.getPrevSort(enterpriseId, storeField.getSort());
            if (needUpdateSort == null) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "已经是第一条，不能上移");
            }
        } else {
            needUpdateSort = storeFieldService.getNextSort(enterpriseId, storeField.getSort());
            if (needUpdateSort == null) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "已经是最后一条，不能下移");
            }
        }
        storeFieldService.updateSortById(needUpdateSort.getStoreFieldId(), storeField.getSort());
        storeFieldService.updateSortById(storeField.getStoreFieldId(), needUpdateSort.getSort());
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<String> setStoreFieldSort(int storeFieldId, int setValue, Integer enterpriseId) {
        TabStoreField storeField = storeFieldService.getStoreFieldById(storeFieldId);
        if (storeField == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "主键错误，查询不到数据");
        }
        storeFieldService.setStoreFieldSort(storeFieldId, enterpriseId, setValue);
        return ServiceResponse.success(storeField.getStoreFieldName());
    }

    @Override
    public ServiceResponse<String> delete(int storeFieldId) {
        TabStoreField storeField = storeFieldService.getStoreFieldById(storeFieldId);
        if (storeField == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "主键错误，查询不到数据");
        }
        if (storeExtendService.storeExtendHasStoreField(storeFieldId)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(),
                    "属性 " + storeField.getStoreFieldName() + " 在使用中，不允许删除");
        }
        storeFieldRegionRelService.deleteByStoreFieldId(storeFieldId);
        storeFieldService.delete(storeFieldId);
        return ServiceResponse.success(storeField.getStoreFieldName());
    }

    @Override
    public ServiceResponse<Integer> deleteAll(int enterpriseId) {
        List<StoreFieldDTO> fieldList = storeFieldService.listStoreFieldByEnterpriseId(enterpriseId);
        List<Integer> storeFieldIdList = new ArrayList<>();
        StringBuilder resultMessage = new StringBuilder();
        for (StoreFieldDTO field : fieldList) {
            storeFieldIdList.add(field.getStoreFieldId());
            if (storeExtendService.storeExtendHasStoreField(field.getStoreFieldId())) {
                resultMessage.append("属性 ").append(field.getStoreFieldName()).append(" 在使用中，不允许删除；");
            }
        }
        if (StringUtils.isNotBlank(resultMessage.toString())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), resultMessage.toString());
        }
        storeFieldRegionRelService.deleteFetchByStoreFieldIds(storeFieldIdList);
        return ServiceResponse.success(storeFieldService.deleteAll(enterpriseId));
    }

    @Override
    public ServiceResponse<String> deleteBatch(List<Integer> storeFieldIds) {
        StringBuilder string = new StringBuilder();

        List<TabStoreField> fieldList = storeFieldService.listByBatchId(storeFieldIds);
        StringBuilder resultMessage = new StringBuilder();
        for (TabStoreField field : fieldList) {
            string.append(field.getStoreFieldName()).append(SignConstants.COMMA);
            if (storeExtendService.storeExtendHasStoreField(field.getStoreFieldId())) {
                resultMessage.append("属性 ").append(field.getStoreFieldName()).append(" 在使用中，不允许删除；");
            }
        }
        if (StringUtils.isNotBlank(resultMessage.toString())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), resultMessage.toString());
        }
        storeFieldRegionRelService.deleteFetchByStoreFieldIds(storeFieldIds);
        storeFieldService.deleteBatch(storeFieldIds);


        return ServiceResponse.success(string.toString());
    }

    @Override
    public ServiceResponse<Integer> edit(StoreFieldDTO storeFieldDTO) {
        ServiceResponse validParam = ValidUtil.allCheckValidate(storeFieldDTO, StoreFieldDTO.EditFieldValidView.class);
        if (!validParam.isSuccess()) {
            return validParam;
        }

        TabStoreField storeField = storeFieldService.getStoreFieldById(storeFieldDTO.getStoreFieldId());
        if (storeField == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "主键错误，查询不到数据");
        }

        //判断是否是owner，如果不是，则不允许编辑
        TabStoreFieldRel tabStoreFieldRel = storeFieldRelService
                .getByEnterpriseIdAndStoreFieldId(storeFieldDTO.getEnterpriseId(), storeFieldDTO.getStoreFieldId());
        if (tabStoreFieldRel.getOwnType().intValue() == StoreOwnTypeEnum.OTHER.getCode()) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "被授权字段，不可编辑");
        }

        if (storeFieldService.isRepeatName(storeFieldDTO.getEnterpriseId(), storeFieldDTO.getStoreFieldId(),
                storeFieldDTO.getStoreFieldName())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "属性名称不能重复");
        }

        String[] regionArr = storeFieldDTO.getStoreRegionId().split("_");
        for (String region : regionArr) {
            if (StringUtils.isBlank(region)) {
                continue;
            }
            if (!NumberUtils.isNumber(region)) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "门店域有误");
            }
            if (storeRegionService.getById(storeFieldDTO.getEnterpriseId(), Integer.parseInt(region)) == null) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "门店域有误");
            }
        }

        //字段类型不能修改
        storeFieldDTO.setStoreFieldType(storeField.getStoreFieldType());

        ServiceResponse validParamResult = StoreFieldValidParamUtil
                .validStoreFieldDetail(storeFieldDTO.getStoreFieldType(), storeFieldDTO.getStoreFieldDetail());
        if (!validParamResult.isSuccess()) {
            return validParamResult;
        }

        //更新字段和门店域关联表数据
        List<StoreFieldRegionRelDTO> regionRelDTOList = new ArrayList<>();
        Date now = new Date();
        for (String regionId : regionArr) {
            if (StringUtils.isBlank(regionId)) {
                continue;
            }
            StoreFieldRegionRelDTO fieldRegionRelDTO = new StoreFieldRegionRelDTO();
            fieldRegionRelDTO.setCreateTime(now);
            fieldRegionRelDTO.setUpdateTime(now);
            fieldRegionRelDTO.setDeleteFlag(0);
            fieldRegionRelDTO.setRegionId(Integer.parseInt(regionId));
            fieldRegionRelDTO.setStoreFieldId(storeFieldDTO.getStoreFieldId());
            regionRelDTOList.add(fieldRegionRelDTO);
        }
        //新增门店域门店属性光关联
        storeFieldRegionRelService.deleteByStoreFieldId(storeFieldDTO.getStoreFieldId());
        storeFieldRegionRelService.saveFetch(regionRelDTOList);

        storeFieldDTO.setStoreFieldCode(storeField.getStoreFieldCode());
        storeFieldDTO.setCreateTime(storeField.getCreateTime());
        storeFieldDTO.setUpdateTime(new Date());
        storeFieldDTO.setErpRelCode(storeField.getErpRelCode());
        return ServiceResponse.success(storeFieldService.edit(storeFieldDTO));
    }

    @Override
    public ServiceResponse<Page<StoreFieldDTO>> listStoreField(int enterpriseId, String search, Integer pageNum,
            Integer pageSize) {
        //查询有效的域ID list
        List<Integer> regionIdList = storeRegionService.listStoreRegion(enterpriseId);
        //根据域查询字段ID list
        List<Integer> storeFieldIdList = storeFieldRegionRelService.listStoreFieldId(regionIdList);

        return listStoreField(storeFieldIdList, enterpriseId, search, pageNum, pageSize);
    }

    @Override
    public ServiceResponse<Page<StoreFieldDTO>> listStoreField(List<Integer> storeFieldIdList, int enterpriseId,
            String search, Integer pageNum, Integer pageSize) {
        com.github.pagehelper.Page storeFieldList = storeFieldService.listStoreField(enterpriseId, search, pageNum,
                pageSize, storeFieldIdList);
        Page<StoreFieldDTO> page = PageHelperUtils.changePageHelperToCurrentPage(storeFieldList);
        List<StoreFieldDTO> selectDTOList = page.getResult();
        int index = (pageNum - 1) * pageSize + 1;
        for (int i = 0, length = selectDTOList.size(); i < length; i++) {
            selectDTOList.get(i).setSeq(index + i);
        }
        return ServiceResponse.success(page);
    }

    @Override
    public ServiceResponse<Map<Integer, String>> storeFieldIdMapRegionId(Integer enterpriseId) {
        //查询有效的域ID list
        List<Integer> regionIdList = storeRegionService.listStoreRegion(enterpriseId);
        //根据域查询字段ID list
        List<Integer> storeFieldIdList = storeFieldRegionRelService.listStoreFieldId(regionIdList);
        List<StoreFieldRegionMapDTO> list = storeFieldRegionRelService.listByStoreFieldIdList(storeFieldIdList);
        Map<Integer, String> map = new HashMap<>(16);
        for (StoreFieldRegionMapDTO storeFieldRegionMapDTO : list) {
            map.put(storeFieldRegionMapDTO.getStoreFieldId(), storeFieldRegionMapDTO.getRegionIds());
        }
        return ServiceResponse.success(map);
    }

    @Override
    public ServiceResponse<List<StoreFieldDTO>> listStoreField(Integer enterpriseId, String search) {
        //查询有效的域ID list
        List<Integer> regionIdList = storeRegionService.listStoreRegion(enterpriseId);
        //根据域查询字段ID list
        List<Integer> storeFieldIdList = storeFieldRegionRelService.listStoreFieldId(regionIdList);
        return ServiceResponse.success(storeFieldService.listStoreField(enterpriseId, search, storeFieldIdList));
    }

    @Override
    public ServiceResponse<StoreFieldDTO> getStoreField(int storeFieldId) {
        TabStoreField tab = storeFieldService.getStoreFieldById(storeFieldId);
        if (tab == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "主键ID有误");
        }
        return ServiceResponse.success(EntityUtil.changeEntityNew(StoreFieldDTO.class, tab));
    }

    @Override
    public ServiceResponse<Integer> sortBottom(int storeFieldId, Integer enterpriseId) {
        TabStoreField storeField = storeFieldService.getStoreFieldById(storeFieldId);
        if (storeField == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "主键错误，查询不到数据");
        }
        storeFieldService.sortBottom(enterpriseId, storeFieldId);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Integer> sortTopping(int storeFieldId, Integer enterpriseId) {
        TabStoreField storeField = storeFieldService.getStoreFieldById(storeFieldId);
        if (storeField == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "主键错误，查询不到数据");
        }
        storeFieldService.sortTopping(enterpriseId, storeFieldId);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<List<StoreFieldDTO>> listStoreFieldByRegionId(Integer regionId) {
        List<StoreFieldDTO> tabStoreFieldList = storeFieldService.listStoreFieldByRegionId(regionId);

        if (tabStoreFieldList == null || tabStoreFieldList.size() < 1) {
            return ServiceResponse.success(new ArrayList<>());
        } else {
            return ServiceResponse.success(tabStoreFieldList);
        }
    }

    @Override
    public ServiceResponse<Void> authStoreField(Integer fromEnterpriseId, Integer toEnterpriseId, List<Integer> regionIdList) {
        //查询域下所有的字段值ID
        List<Integer> storeFieldIdList = storeFieldRegionRelService.listStoreFieldId(regionIdList);
        //查询授权商户下所有的owner 自定义字段值
        List<StoreFieldDTO> storeFieldList = storeFieldService.listOwnStoreField(fromEnterpriseId, storeFieldIdList);
        if (storeFieldList == null || storeFieldList.size() < 1) {
            LOGGER.info("授权自定义字段，无数据授权");
            return ServiceResponse.success();
        }
        List<Integer> ownStoreFieldList = new ArrayList<>(storeFieldList.size());
        for (StoreFieldDTO storeFieldDTO : storeFieldList) {
            ownStoreFieldList.add(storeFieldDTO.getStoreFieldId());
        }
        //delete 被授权的，避免重复授权
        storeFieldRelService.deleteFetch(ownStoreFieldList, toEnterpriseId);
        //新增
        List<StoreFieldRelDTO> storeFieldRelList = new ArrayList<>(ownStoreFieldList.size());
        Date now = new Date();
        for (Integer ownStoreField : ownStoreFieldList) {
            StoreFieldRelDTO dto = new StoreFieldRelDTO();
            dto.setCreateTime(now);
            dto.setUpdateTime(now);
            dto.setStoreFieldId(ownStoreField);
            dto.setEnterpriseId(toEnterpriseId);
            dto.setOwnType(StoreOwnTypeEnum.OTHER.getCode());
            dto.setDeleteFlag(0);
            storeFieldRelList.add(dto);
        }
        storeFieldRelService.saveFetch(storeFieldRelList);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<List<StoreFieldDTO>> listOwnStoreField(Integer enterpriseId) {
        //查询有效的域ID list
        List<Integer> regionIdList = storeRegionService.listStoreRegion(enterpriseId);
        //根据域查询字段ID list
        List<Integer> storeFieldIdList = storeFieldRegionRelService.listStoreFieldId(regionIdList);
        //查询商户下所有的owner 自定义字段值
        List<StoreFieldDTO> storeFieldList = storeFieldService.listOwnStoreField(enterpriseId, storeFieldIdList);
        return ServiceResponse.success(storeFieldList);
    }

    @Override
    public ServiceResponse<Void> setErpMapperCode(Integer storeFieldId, String code) {
        TabStoreField storeField = storeFieldService.getStoreFieldById(storeFieldId);
        if (storeField == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "主键错误，查询不到数据");
        }
        StoreFieldDTO dto = new StoreFieldDTO();
        dto.setStoreFieldId(storeFieldId);
        //表明已经做关联
        dto.setErpRelCode(code);
        storeFieldService.edit(dto);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> unBindErpMapper(Integer storeFieldId) {
        TabStoreField storeField = storeFieldService.getStoreFieldById(storeFieldId);
        if (storeField == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "主键错误，查询不到数据");
        }
        StoreFieldDTO dto = new StoreFieldDTO();
        dto.setStoreFieldId(storeFieldId);
        //表明已经解除关联
        dto.setErpRelCode("");
        storeFieldService.edit(dto);
        return ServiceResponse.success();
    }
}
